# vim: set fileencoding=utf-8 :

from __future__ import unicode_literals
from concrete.validate import validate_communication

from concrete.util import (
    create_comm, create_simple_comm,
    AL_NONE, AL_SECTION, AL_SENTENCE
)


def test_create_simple_comm():
    comm = create_simple_comm('one')
    assert 'one' == comm.id
    assert 'Super simple sentence .' == comm.text
    assert validate_communication(comm)


def test_create_comm_empty():
    comm = create_comm('one')
    assert 'one' == comm.id
    assert '' == comm.text
    assert [] == comm.sectionList
    assert validate_communication(comm)


def test_create_comm_ws():
    comm = create_comm('one', '\t \t\r\n\n')
    assert 'one' == comm.id
    assert '\t \t\r\n\n' == comm.text
    assert [] == comm.sectionList
    assert validate_communication(comm)


def test_create_comm_unicode():
    comm = create_comm('one', u'狐狸\t\t.')
    assert 'one' == comm.id
    assert u'狐狸\t\t.' == comm.text
    assert 1 == len(comm.sectionList)
    sect = comm.sectionList[0]
    assert 0 == sect.textSpan.start
    assert 5 == sect.textSpan.ending
    assert 1 == len(sect.sentenceList)
    sent = sect.sentenceList[0]
    assert 0 == sent.textSpan.start
    assert 5 == sent.textSpan.ending
    tl = sent.tokenization.tokenList.tokenList
    assert 2 == len(tl)
    assert 0 == tl[0].tokenIndex
    assert u'狐狸' == tl[0].text
    assert u'狐狸' == comm.text[tl[0].textSpan.start:tl[0].textSpan.ending]
    assert 1 == tl[1].tokenIndex
    assert '.' == tl[1].text
    assert validate_communication(comm)


def test_create_comm_one_sentence():
    comm = create_comm('one', 'simple comm\t\t.')
    assert 'one' == comm.id
    assert 'simple comm\t\t.' == comm.text
    assert 1 == len(comm.sectionList)
    sect = comm.sectionList[0]
    assert 0 == sect.textSpan.start
    assert 14 == sect.textSpan.ending
    assert 1 == len(sect.sentenceList)
    sent = sect.sentenceList[0]
    assert 0 == sent.textSpan.start
    assert 14 == sent.textSpan.ending
    tl = sent.tokenization.tokenList.tokenList
    assert 3 == len(tl)
    assert 0 == tl[0].tokenIndex
    assert 'simple' == tl[0].text
    assert 'simple' == comm.text[tl[0].textSpan.start:tl[0].textSpan.ending]
    assert 1 == tl[1].tokenIndex
    assert 'comm' == tl[1].text
    assert 'comm' == comm.text[tl[1].textSpan.start:tl[1].textSpan.ending]
    assert 2 == tl[2].tokenIndex
    assert '.' == tl[2].text
    assert '.' == comm.text[tl[2].textSpan.start:tl[2].textSpan.ending]
    assert validate_communication(comm)


def test_create_comm_complex():
    comm = create_comm('one', '\n\nsimple comm\t\t.\nor ...\n\nisit?\n')
    assert 'one' == comm.id
    assert '\n\nsimple comm\t\t.\nor ...\n\nisit?\n' == comm.text
    assert 2 == len(comm.sectionList)

    sect = comm.sectionList[0]
    assert 2 == sect.textSpan.start
    assert 23 == sect.textSpan.ending
    assert 2 == len(sect.sentenceList)
    sent = sect.sentenceList[0]
    assert 2 == sent.textSpan.start
    assert 16 == sent.textSpan.ending
    tl = sent.tokenization.tokenList.tokenList
    assert 3 == len(tl)
    assert 0 == tl[0].tokenIndex
    assert 'simple' == tl[0].text
    assert 'simple' == comm.text[tl[0].textSpan.start:tl[0].textSpan.ending]
    assert 1 == tl[1].tokenIndex
    assert 'comm' == tl[1].text
    assert 'comm' == comm.text[tl[1].textSpan.start:tl[1].textSpan.ending]
    assert 2 == tl[2].tokenIndex
    assert '.' == tl[2].text
    assert '.' == comm.text[tl[2].textSpan.start:tl[2].textSpan.ending]
    sent = sect.sentenceList[1]
    assert 17 == sent.textSpan.start
    assert 23 == sent.textSpan.ending
    tl = sent.tokenization.tokenList.tokenList
    assert 2 == len(tl)
    assert 0 == tl[0].tokenIndex
    assert 'or' == tl[0].text
    assert 1 == tl[1].tokenIndex
    assert '...' == tl[1].text

    sect = comm.sectionList[1]
    assert 25 == sect.textSpan.start
    assert 30 == sect.textSpan.ending
    assert 1 == len(sect.sentenceList)
    sent = sect.sentenceList[0]
    assert 25 == sent.textSpan.start
    assert 30 == sent.textSpan.ending
    tl = sent.tokenization.tokenList.tokenList
    assert 1 == len(tl)
    assert 0 == tl[0].tokenIndex
    assert 'isit?' == tl[0].text
    assert 'isit?' == comm.text[tl[0].textSpan.start:tl[0].textSpan.ending]

    assert validate_communication(comm)


def test_create_comm_complex_sections():
    comm = create_comm('one', '\n\n\nFOO\r\n\r\n\n\n\nBAR\n\nFU\nBAR\n\t\n\n   \n')
    assert 'one' == comm.id
    assert '\n\n\nFOO\r\n\r\n\n\n\nBAR\n\nFU\nBAR\n\t\n\n   \n' == comm.text
    assert 3 == len(comm.sectionList)

    sect = comm.sectionList[0]
    assert 3 == sect.textSpan.start
    assert 6 == sect.textSpan.ending
    assert 1 == len(sect.sentenceList)

    sect = comm.sectionList[1]
    assert 13 == sect.textSpan.start
    assert 16 == sect.textSpan.ending
    assert 1 == len(sect.sentenceList)

    sect = comm.sectionList[2]
    assert 18 == sect.textSpan.start
    assert 24 == sect.textSpan.ending
    assert 2 == len(sect.sentenceList)


def test_create_comm_empty_al_none():
    comm = create_comm('one', annotation_level=AL_NONE)
    assert 'one' == comm.id
    assert '' == comm.text
    assert comm.sectionList is None
    assert validate_communication(comm)


def test_create_comm_ws_al_none():
    comm = create_comm('one', '\t \t\r\n\n', annotation_level=AL_NONE)
    assert 'one' == comm.id
    assert '\t \t\r\n\n' == comm.text
    assert comm.sectionList is None
    assert validate_communication(comm)


def test_create_comm_unicode_al_none():
    comm = create_comm('one', u'狐狸\t\t.', annotation_level=AL_NONE)
    assert 'one' == comm.id
    assert u'狐狸\t\t.' == comm.text
    assert comm.sectionList is None
    assert validate_communication(comm)


def test_create_comm_one_sentence_al_none():
    comm = create_comm('one', 'simple comm\t\t.', annotation_level=AL_NONE)
    assert 'one' == comm.id
    assert 'simple comm\t\t.' == comm.text
    assert comm.sectionList is None
    assert validate_communication(comm)


def test_create_comm_complex_al_none():
    comm = create_comm('one', '\n\nsimple comm\t\t.\nor ...\n\nisit?\n',
                       annotation_level=AL_NONE)
    assert 'one' == comm.id
    assert '\n\nsimple comm\t\t.\nor ...\n\nisit?\n' == comm.text
    assert comm.sectionList is None
    assert validate_communication(comm)


def test_create_comm_empty_al_section():
    comm = create_comm('one', annotation_level=AL_SECTION)
    assert 'one' == comm.id
    assert '' == comm.text
    assert [] == comm.sectionList
    assert validate_communication(comm)


def test_create_comm_ws_al_section():
    comm = create_comm('one', '\t \t\r\n\n', annotation_level=AL_SECTION)
    assert 'one' == comm.id
    assert '\t \t\r\n\n' == comm.text
    assert [] == comm.sectionList
    assert validate_communication(comm)


def test_create_comm_unicode_al_section():
    comm = create_comm('one', u'狐狸\t\t.', annotation_level=AL_SECTION)
    assert 'one' == comm.id
    assert u'狐狸\t\t.' == comm.text
    assert 1 == len(comm.sectionList)
    sect = comm.sectionList[0]
    assert 0 == sect.textSpan.start
    assert 5 == sect.textSpan.ending
    assert sect.sentenceList is None
    assert validate_communication(comm)


def test_create_comm_one_sentence_al_section():
    comm = create_comm('one', 'simple comm\t\t.', annotation_level=AL_SECTION)
    assert 'one' == comm.id
    assert 'simple comm\t\t.' == comm.text
    assert 1 == len(comm.sectionList)
    sect = comm.sectionList[0]
    assert 0 == sect.textSpan.start
    assert 14 == sect.textSpan.ending
    assert sect.sentenceList is None
    assert validate_communication(comm)


def test_create_comm_complex_al_section():
    comm = create_comm('one', '\n\nsimple comm\t\t.\nor ...\n\nisit?\n',
                       annotation_level=AL_SECTION)
    assert 'one' == comm.id
    assert '\n\nsimple comm\t\t.\nor ...\n\nisit?\n' == comm.text

    assert 2 == len(comm.sectionList)

    sect = comm.sectionList[0]
    assert 2 == sect.textSpan.start
    assert 23 == sect.textSpan.ending
    assert sect.sentenceList is None

    sect = comm.sectionList[1]
    assert 25 == sect.textSpan.start
    assert 30 == sect.textSpan.ending
    assert sect.sentenceList is None

    assert validate_communication(comm)


def test_create_comm_empty_al_sentence():
    comm = create_comm('one', annotation_level=AL_SENTENCE)
    assert 'one' == comm.id
    assert '' == comm.text
    assert [] == comm.sectionList
    assert validate_communication(comm)


def test_create_comm_ws_al_sentence():
    comm = create_comm('one', '\t \t\r\n\n', annotation_level=AL_SENTENCE)
    assert 'one' == comm.id
    assert '\t \t\r\n\n' == comm.text
    assert [] == comm.sectionList
    assert validate_communication(comm)


def test_create_comm_unicode_al_sentence():
    comm = create_comm('one', u'狐狸\t\t.', annotation_level=AL_SENTENCE)
    assert 'one' == comm.id
    assert u'狐狸\t\t.' == comm.text
    assert 1 == len(comm.sectionList)
    sect = comm.sectionList[0]
    assert 0 == sect.textSpan.start
    assert 5 == sect.textSpan.ending
    assert 1 == len(sect.sentenceList)
    sent = sect.sentenceList[0]
    assert 0 == sent.textSpan.start
    assert 5 == sent.textSpan.ending
    assert sent.tokenization is None
    assert validate_communication(comm)


def test_create_comm_one_sentence_al_sentence():
    comm = create_comm('one', 'simple comm\t\t.', annotation_level=AL_SENTENCE)
    assert 'one' == comm.id
    assert 'simple comm\t\t.' == comm.text
    assert 1 == len(comm.sectionList)
    sect = comm.sectionList[0]
    assert 0 == sect.textSpan.start
    assert 14 == sect.textSpan.ending
    assert 1 == len(sect.sentenceList)
    sent = sect.sentenceList[0]
    assert 0 == sent.textSpan.start
    assert 14 == sent.textSpan.ending
    assert sent.tokenization is None
    assert validate_communication(comm)


def test_create_comm_complex_al_sentence():
    comm = create_comm('one', '\n\nsimple comm\t\t.\nor ...\n\nisit?\n',
                       annotation_level=AL_SENTENCE)
    assert 'one' == comm.id
    assert '\n\nsimple comm\t\t.\nor ...\n\nisit?\n' == comm.text
    assert 2 == len(comm.sectionList)

    sect = comm.sectionList[0]
    assert 2 == sect.textSpan.start
    assert 23 == sect.textSpan.ending
    assert 2 == len(sect.sentenceList)
    sent = sect.sentenceList[0]
    assert 2 == sent.textSpan.start
    assert 16 == sent.textSpan.ending
    assert sent.tokenization is None
    sent = sect.sentenceList[1]
    assert 17 == sent.textSpan.start
    assert 23 == sent.textSpan.ending
    assert sent.tokenization is None

    sect = comm.sectionList[1]
    assert 25 == sect.textSpan.start
    assert 30 == sect.textSpan.ending
    assert 1 == len(sect.sentenceList)
    sent = sect.sentenceList[0]
    assert 25 == sent.textSpan.start
    assert 30 == sent.textSpan.ending
    assert sent.tokenization is None

    assert validate_communication(comm)



#!/usr/bin/env python

from __future__ import unicode_literals
from argparse import ArgumentParser, ArgumentDefaultsHelpFormatter
import logging

from thrift.protocol.TProtocol import TProtocolException
from thrift.transport.TTransport import TTransportException

import concrete.version
from concrete.util import CommunicationReader, CommunicationWriter, FileType
from concrete.util.annotate_wrapper import (
    AnnotateCommunicationClientWrapper,
    HTTPAnnotateCommunicationClientWrapper
)
from concrete.util import set_stdout_encoding


def main():
    set_stdout_encoding()

    parser = ArgumentParser(
        formatter_class=ArgumentDefaultsHelpFormatter,
        description="Interface with a Concrete AnnotateCommunicationService server. "
        "Supports either THttp/TJSONProtocol (using the '--uri' flag) "
        "or TSocket/TCompactProtocol (using '--host'/'--port')"
    )
    parser.add_argument('--host', default='localhost',
                        help="Hostname of TSocket/TCompactProtocol AnnotateCommunicationService")
    parser.add_argument('-p', '--port', type=int, default=9090,
                        help="Port of TSocket/TCompactProtocol AnnotateCommunicationService")
    parser.add_argument('--uri', '--url',
                        help="URI of THttpServer/TJSONProtocol AnnotateCommunicationService")
    parser.add_argument('-l', '--loglevel', '--log-level',
                        help='Logging verbosity level threshold (to stderr)',
                        default='info')
    parser.add_argument('--input', default='-',
                        help="Input source to use. '-' for stdin; otherwise"
                             " takes a path to a file.")
    parser.add_argument('--output', default='-',
                        help="Output source to use. '-' for stdout; otherwise"
                             " takes a path to a file.")
    concrete.version.add_argparse_argument(parser)
    args = parser.parse_args()

    logging.basicConfig(format='%(asctime)-15s %(levelname)s: %(message)s',
                        level=args.loglevel.upper())

    # Won't work on Windows
    if args.input == '-':
        reader_kwargs = dict(filetype=FileType.STREAM)
        input_path = '/dev/fd/0'
    else:
        reader_kwargs = dict()
        input_path = args.input
    output_path = '/dev/fd/1' if args.output == '-' else args.output

    reader = CommunicationReader(input_path, **reader_kwargs)
    if args.uri:
        try:
            with HTTPAnnotateCommunicationClientWrapper(args.uri) as client:
                with CommunicationWriter(output_path) as writer:
                    for (comm, _) in reader:
                        writer.write(client.annotate(comm))
        except TProtocolException as ex:
            logging.error(ex)
            logging.error(
                "Successfully connected to the URI '{}' using HTTP, but the URI does not "
                "appear to be an AnnotateCommunicationService endpoint that uses the "
                "Thrift THttp transport and TJSONProtocol encoding".format(args.uri))
    else:
        try:
            with AnnotateCommunicationClientWrapper(args.host, args.port) as client:
                with CommunicationWriter(output_path) as writer:
                    for (comm, _) in reader:
                        writer.write(client.annotate(comm))
        except TTransportException:
            pass


if __name__ == "__main__":
    main()



# -*- coding: utf-8 -*-
#
# Autogenerated by Thrift Compiler (0.16.0)
#
# DO NOT EDIT UNLESS YOU ARE SURE THAT YOU KNOW WHAT YOU ARE DOING
#
#  options string: py:coding=utf-8
#

from thrift.Thrift import TType, TMessageType, TFrozenDict, TException, TApplicationException
from thrift.protocol.TProtocol import TProtocolException
from thrift.TRecursive import fix_spec

import sys
import concrete.services.Service
import logging
from .ttypes import *
from thrift.Thrift import TProcessor
from thrift.transport import TTransport
all_structs = []


class Iface(concrete.services.Service.Iface):
    def registerSearchResult(self, result, taskType):
        """
        Register the specified search result for annotation.

        If a name has not been assigned to the search query, one will be generated.
        This service also requires that the user_id field be populated in the SearchQuery.

        Parameters:
         - result
         - taskType

        """
        pass

    def getSearchResults(self, taskType, limit):
        """
        Get a list of search results for a particular annotation task
        Set the limit to 0 to get all relevant search results

        Parameters:
         - taskType
         - limit

        """
        pass

    def getSearchResultsByUser(self, taskType, userId, limit):
        """
        Get a list of search results for a particular annotation task filtered by a user id
        Set the limit to 0 to get all relevant search results

        Parameters:
         - taskType
         - userId
         - limit

        """
        pass

    def getLatestSearchResult(self, userId):
        """
        Get the most recent search results for a user

        Parameters:
         - userId

        """
        pass

    def getSearchResult(self, searchResultId):
        """
        Get a search result object

        Parameters:
         - searchResultId

        """
        pass

    def startSession(self, searchResultId, taskType):
        """
        Start an annotation session
        Returns a session id used in future session calls

        Parameters:
         - searchResultId
         - taskType

        """
        pass

    def stopSession(self, sessionId):
        """
        Stops an annotation session

        Parameters:
         - sessionId

        """
        pass

    def getNextChunk(self, sessionId):
        """
        Get next chunk of data to annotate
        The client should use the Retriever service to access the data

        Parameters:
         - sessionId

        """
        pass

    def submitAnnotation(self, sessionId, unitId, communication):
        """
        Submit an annotation for a session

        Parameters:
         - sessionId
         - unitId
         - communication

        """
        pass


class Client(concrete.services.Service.Client, Iface):
    def __init__(self, iprot, oprot=None):
        concrete.services.Service.Client.__init__(self, iprot, oprot)

    def registerSearchResult(self, result, taskType):
        """
        Register the specified search result for annotation.

        If a name has not been assigned to the search query, one will be generated.
        This service also requires that the user_id field be populated in the SearchQuery.

        Parameters:
         - result
         - taskType

        """
        self.send_registerSearchResult(result, taskType)
        self.recv_registerSearchResult()

    def send_registerSearchResult(self, result, taskType):
        self._oprot.writeMessageBegin('registerSearchResult', TMessageType.CALL, self._seqid)
        args = registerSearchResult_args()
        args.result = result
        args.taskType = taskType
        args.write(self._oprot)
        self._oprot.writeMessageEnd()
        self._oprot.trans.flush()

    def recv_registerSearchResult(self):
        iprot = self._iprot
        (fname, mtype, rseqid) = iprot.readMessageBegin()
        if mtype == TMessageType.EXCEPTION:
            x = TApplicationException()
            x.read(iprot)
            iprot.readMessageEnd()
            raise x
        result = registerSearchResult_result()
        result.read(iprot)
        iprot.readMessageEnd()
        if result.ex is not None:
            raise result.ex
        return

    def getSearchResults(self, taskType, limit):
        """
        Get a list of search results for a particular annotation task
        Set the limit to 0 to get all relevant search results

        Parameters:
         - taskType
         - limit

        """
        self.send_getSearchResults(taskType, limit)
        return self.recv_getSearchResults()

    def send_getSearchResults(self, taskType, limit):
        self._oprot.writeMessageBegin('getSearchResults', TMessageType.CALL, self._seqid)
        args = getSearchResults_args()
        args.taskType = taskType
        args.limit = limit
        args.write(self._oprot)
        self._oprot.writeMessageEnd()
        self._oprot.trans.flush()

    def recv_getSearchResults(self):
        iprot = self._iprot
        (fname, mtype, rseqid) = iprot.readMessageBegin()
        if mtype == TMessageType.EXCEPTION:
            x = TApplicationException()
            x.read(iprot)
            iprot.readMessageEnd()
            raise x
        result = getSearchResults_result()
        result.read(iprot)
        iprot.readMessageEnd()
        if result.success is not None:
            return result.success
        if result.ex is not None:
            raise result.ex
        raise TApplicationException(TApplicationException.MISSING_RESULT, "getSearchResults failed: unknown result")

    def getSearchResultsByUser(self, taskType, userId, limit):
        """
        Get a list of search results for a particular annotation task filtered by a user id
        Set the limit to 0 to get all relevant search results

        Parameters:
         - taskType
         - userId
         - limit

        """
        self.send_getSearchResultsByUser(taskType, userId, limit)
        return self.recv_getSearchResultsByUser()

    def send_getSearchResultsByUser(self, taskType, userId, limit):
        self._oprot.writeMessageBegin('getSearchResultsByUser', TMessageType.CALL, self._seqid)
        args = getSearchResultsByUser_args()
        args.taskType = taskType
        args.userId = userId
        args.limit = limit
        args.write(self._oprot)
        self._oprot.writeMessageEnd()
        self._oprot.trans.flush()

    def recv_getSearchResultsByUser(self):
        iprot = self._iprot
        (fname, mtype, rseqid) = iprot.readMessageBegin()
        if mtype == TMessageType.EXCEPTION:
            x = TApplicationException()
            x.read(iprot)
            iprot.readMessageEnd()
            raise x
        result = getSearchResultsByUser_result()
        result.read(iprot)
        iprot.readMessageEnd()
        if result.success is not None:
            return result.success
        if result.ex is not None:
            raise result.ex
        raise TApplicationException(TApplicationException.MISSING_RESULT, "getSearchResultsByUser failed: unknown result")

    def getLatestSearchResult(self, userId):
        """
        Get the most recent search results for a user

        Parameters:
         - userId

        """
        self.send_getLatestSearchResult(userId)
        return self.recv_getLatestSearchResult()

    def send_getLatestSearchResult(self, userId):
        self._oprot.writeMessageBegin('getLatestSearchResult', TMessageType.CALL, self._seqid)
        args = getLatestSearchResult_args()
        args.userId = userId
        args.write(self._oprot)
        self._oprot.writeMessageEnd()
        self._oprot.trans.flush()

    def recv_getLatestSearchResult(self):
        iprot = self._iprot
        (fname, mtype, rseqid) = iprot.readMessageBegin()
        if mtype == TMessageType.EXCEPTION:
            x = TApplicationException()
            x.read(iprot)
            iprot.readMessageEnd()
            raise x
        result = getLatestSearchResult_result()
        result.read(iprot)
        iprot.readMessageEnd()
        if result.success is not None:
            return result.success
        if result.ex is not None:
            raise result.ex
        raise TApplicationException(TApplicationException.MISSING_RESULT, "getLatestSearchResult failed: unknown result")

    def getSearchResult(self, searchResultId):
        """
        Get a search result object

        Parameters:
         - searchResultId

        """
        self.send_getSearchResult(searchResultId)
        return self.recv_getSearchResult()

    def send_getSearchResult(self, searchResultId):
        self._oprot.writeMessageBegin('getSearchResult', TMessageType.CALL, self._seqid)
        args = getSearchResult_args()
        args.searchResultId = searchResultId
        args.write(self._oprot)
        self._oprot.writeMessageEnd()
        self._oprot.trans.flush()

    def recv_getSearchResult(self):
        iprot = self._iprot
        (fname, mtype, rseqid) = iprot.readMessageBegin()
        if mtype == TMessageType.EXCEPTION:
            x = TApplicationException()
            x.read(iprot)
            iprot.readMessageEnd()
            raise x
        result = getSearchResult_result()
        result.read(iprot)
        iprot.readMessageEnd()
        if result.success is not None:
            return result.success
        if result.ex is not None:
            raise result.ex
        raise TApplicationException(TApplicationException.MISSING_RESULT, "getSearchResult failed: unknown result")

    def startSession(self, searchResultId, taskType):
        """
        Start an annotation session
        Returns a session id used in future session calls

        Parameters:
         - searchResultId
         - taskType

        """
        self.send_startSession(searchResultId, taskType)
        return self.recv_startSession()

    def send_startSession(self, searchResultId, taskType):
        self._oprot.writeMessageBegin('startSession', TMessageType.CALL, self._seqid)
        args = startSession_args()
        args.searchResultId = searchResultId
        args.taskType = taskType
        args.write(self._oprot)
        self._oprot.writeMessageEnd()
        self._oprot.trans.flush()

    def recv_startSession(self):
        iprot = self._iprot
        (fname, mtype, rseqid) = iprot.readMessageBegin()
        if mtype == TMessageType.EXCEPTION:
            x = TApplicationException()
            x.read(iprot)
            iprot.readMessageEnd()
            raise x
        result = startSession_result()
        result.read(iprot)
        iprot.readMessageEnd()
        if result.success is not None:
            return result.success
        if result.ex is not None:
            raise result.ex
        raise TApplicationException(TApplicationException.MISSING_RESULT, "startSession failed: unknown result")

    def stopSession(self, sessionId):
        """
        Stops an annotation session

        Parameters:
         - sessionId

        """
        self.send_stopSession(sessionId)
        self.recv_stopSession()

    def send_stopSession(self, sessionId):
        self._oprot.writeMessageBegin('stopSession', TMessageType.CALL, self._seqid)
        args = stopSession_args()
        args.sessionId = sessionId
        args.write(self._oprot)
        self._oprot.writeMessageEnd()
        self._oprot.trans.flush()

    def recv_stopSession(self):
        iprot = self._iprot
        (fname, mtype, rseqid) = iprot.readMessageBegin()
        if mtype == TMessageType.EXCEPTION:
            x = TApplicationException()
            x.read(iprot)
            iprot.readMessageEnd()
            raise x
        result = stopSession_result()
        result.read(iprot)
        iprot.readMessageEnd()
        if result.ex is not None:
            raise result.ex
        return

    def getNextChunk(self, sessionId):
        """
        Get next chunk of data to annotate
        The client should use the Retriever service to access the data

        Parameters:
         - sessionId

        """
        self.send_getNextChunk(sessionId)
        return self.recv_getNextChunk()

    def send_getNextChunk(self, sessionId):
        self._oprot.writeMessageBegin('getNextChunk', TMessageType.CALL, self._seqid)
        args = getNextChunk_args()
        args.sessionId = sessionId
        args.write(self._oprot)
        self._oprot.writeMessageEnd()
        self._oprot.trans.flush()

    def recv_getNextChunk(self):
        iprot = self._iprot
        (fname, mtype, rseqid) = iprot.readMessageBegin()
        if mtype == TMessageType.EXCEPTION:
            x = TApplicationException()
            x.read(iprot)
            iprot.readMessageEnd()
            raise x
        result = getNextChunk_result()
        result.read(iprot)
        iprot.readMessageEnd()
        if result.success is not None:
            return result.success
        if result.ex is not None:
            raise result.ex
        raise TApplicationException(TApplicationException.MISSING_RESULT, "getNextChunk failed: unknown result")

    def submitAnnotation(self, sessionId, unitId, communication):
        """
        Submit an annotation for a session

        Parameters:
         - sessionId
         - unitId
         - communication

        """
        self.send_submitAnnotation(sessionId, unitId, communication)
        self.recv_submitAnnotation()

    def send_submitAnnotation(self, sessionId, unitId, communication):
        self._oprot.writeMessageBegin('submitAnnotation', TMessageType.CALL, self._seqid)
        args = submitAnnotation_args()
        args.sessionId = sessionId
        args.unitId = unitId
        args.communication = communication
        args.write(self._oprot)
        self._oprot.writeMessageEnd()
        self._oprot.trans.flush()

    def recv_submitAnnotation(self):
        iprot = self._iprot
        (fname, mtype, rseqid) = iprot.readMessageBegin()
        if mtype == TMessageType.EXCEPTION:
            x = TApplicationException()
            x.read(iprot)
            iprot.readMessageEnd()
            raise x
        result = submitAnnotation_result()
        result.read(iprot)
        iprot.readMessageEnd()
        if result.ex is not None:
            raise result.ex
        return


class Processor(concrete.services.Service.Processor, Iface, TProcessor):
    def __init__(self, handler):
        concrete.services.Service.Processor.__init__(self, handler)
        self._processMap["registerSearchResult"] = Processor.process_registerSearchResult
        self._processMap["getSearchResults"] = Processor.process_getSearchResults
        self._processMap["getSearchResultsByUser"] = Processor.process_getSearchResultsByUser
        self._processMap["getLatestSearchResult"] = Processor.process_getLatestSearchResult
        self._processMap["getSearchResult"] = Processor.process_getSearchResult
        self._processMap["startSession"] = Processor.process_startSession
        self._processMap["stopSession"] = Processor.process_stopSession
        self._processMap["getNextChunk"] = Processor.process_getNextChunk
        self._processMap["submitAnnotation"] = Processor.process_submitAnnotation
        self._on_message_begin = None

    def on_message_begin(self, func):
        self._on_message_begin = func

    def process(self, iprot, oprot):
        (name, type, seqid) = iprot.readMessageBegin()
        if self._on_message_begin:
            self._on_message_begin(name, type, seqid)
        if name not in self._processMap:
            iprot.skip(TType.STRUCT)
            iprot.readMessageEnd()
            x = TApplicationException(TApplicationException.UNKNOWN_METHOD, 'Unknown function %s' % (name))
            oprot.writeMessageBegin(name, TMessageType.EXCEPTION, seqid)
            x.write(oprot)
            oprot.writeMessageEnd()
            oprot.trans.flush()
            return
        else:
            self._processMap[name](self, seqid, iprot, oprot)
        return True

    def process_registerSearchResult(self, seqid, iprot, oprot):
        args = registerSearchResult_args()
        args.read(iprot)
        iprot.readMessageEnd()
        result = registerSearchResult_result()
        try:
            self._handler.registerSearchResult(args.result, args.taskType)
            msg_type = TMessageType.REPLY
        except TTransport.TTransportException:
            raise
        except concrete.services.ttypes.ServicesException as ex:
            msg_type = TMessageType.REPLY
            result.ex = ex
        except TApplicationException as ex:
            logging.exception('TApplication exception in handler')
            msg_type = TMessageType.EXCEPTION
            result = ex
        except Exception:
            logging.exception('Unexpected exception in handler')
            msg_type = TMessageType.EXCEPTION
            result = TApplicationException(TApplicationException.INTERNAL_ERROR, 'Internal error')
        oprot.writeMessageBegin("registerSearchResult", msg_type, seqid)
        result.write(oprot)
        oprot.writeMessageEnd()
        oprot.trans.flush()

    def process_getSearchResults(self, seqid, iprot, oprot):
        args = getSearchResults_args()
        args.read(iprot)
        iprot.readMessageEnd()
        result = getSearchResults_result()
        try:
            result.success = self._handler.getSearchResults(args.taskType, args.limit)
            msg_type = TMessageType.REPLY
        except TTransport.TTransportException:
            raise
        except concrete.services.ttypes.ServicesException as ex:
            msg_type = TMessageType.REPLY
            result.ex = ex
        except TApplicationException as ex:
            logging.exception('TApplication exception in handler')
            msg_type = TMessageType.EXCEPTION
            result = ex
        except Exception:
            logging.exception('Unexpected exception in handler')
            msg_type = TMessageType.EXCEPTION
            result = TApplicationException(TApplicationException.INTERNAL_ERROR, 'Internal error')
        oprot.writeMessageBegin("getSearchResults", msg_type, seqid)
        result.write(oprot)
        oprot.writeMessageEnd()
        oprot.trans.flush()

    def process_getSearchResultsByUser(self, seqid, iprot, oprot):
        args = getSearchResultsByUser_args()
        args.read(iprot)
        iprot.readMessageEnd()
        result = getSearchResultsByUser_result()
        try:
            result.success = self._handler.getSearchResultsByUser(args.taskType, args.userId, args.limit)
            msg_type = TMessageType.REPLY
        except TTransport.TTransportException:
            raise
        except concrete.services.ttypes.ServicesException as ex:
            msg_type = TMessageType.REPLY
            result.ex = ex
        except TApplicationException as ex:
            logging.exception('TApplication exception in handler')
            msg_type = TMessageType.EXCEPTION
            result = ex
        except Exception:
            logging.exception('Unexpected exception in handler')
            msg_type = TMessageType.EXCEPTION
            result = TApplicationException(TApplicationException.INTERNAL_ERROR, 'Internal error')
        oprot.writeMessageBegin("getSearchResultsByUser", msg_type, seqid)
        result.write(oprot)
        oprot.writeMessageEnd()
        oprot.trans.flush()

    def process_getLatestSearchResult(self, seqid, iprot, oprot):
        args = getLatestSearchResult_args()
        args.read(iprot)
        iprot.readMessageEnd()
        result = getLatestSearchResult_result()
        try:
            result.success = self._handler.getLatestSearchResult(args.userId)
            msg_type = TMessageType.REPLY
        except TTransport.TTransportException:
            raise
        except concrete.services.ttypes.ServicesException as ex:
            msg_type = TMessageType.REPLY
            result.ex = ex
        except TApplicationException as ex:
            logging.exception('TApplication exception in handler')
            msg_type = TMessageType.EXCEPTION
            result = ex
        except Exception:
            logging.exception('Unexpected exception in handler')
            msg_type = TMessageType.EXCEPTION
            result = TApplicationException(TApplicationException.INTERNAL_ERROR, 'Internal error')
        oprot.writeMessageBegin("getLatestSearchResult", msg_type, seqid)
        result.write(oprot)
        oprot.writeMessageEnd()
        oprot.trans.flush()

    def process_getSearchResult(self, seqid, iprot, oprot):
        args = getSearchResult_args()
        args.read(iprot)
        iprot.readMessageEnd()
        result = getSearchResult_result()
        try:
            result.success = self._handler.getSearchResult(args.searchResultId)
            msg_type = TMessageType.REPLY
        except TTransport.TTransportException:
            raise
        except concrete.services.ttypes.ServicesException as ex:
            msg_type = TMessageType.REPLY
            result.ex = ex
        except TApplicationException as ex:
            logging.exception('TApplication exception in handler')
            msg_type = TMessageType.EXCEPTION
            result = ex
        except Exception:
            logging.exception('Unexpected exception in handler')
            msg_type = TMessageType.EXCEPTION
            result = TApplicationException(TApplicationException.INTERNAL_ERROR, 'Internal error')
        oprot.writeMessageBegin("getSearchResult", msg_type, seqid)
        result.write(oprot)
        oprot.writeMessageEnd()
        oprot.trans.flush()

    def process_startSession(self, seqid, iprot, oprot):
        args = startSession_args()
        args.read(iprot)
        iprot.readMessageEnd()
        result = startSession_result()
        try:
            result.success = self._handler.startSession(args.searchResultId, args.taskType)
            msg_type = TMessageType.REPLY
        except TTransport.TTransportException:
            raise
        except concrete.services.ttypes.ServicesException as ex:
            msg_type = TMessageType.REPLY
            result.ex = ex
        except TApplicationException as ex:
            logging.exception('TApplication exception in handler')
            msg_type = TMessageType.EXCEPTION
            result = ex
        except Exception:
            logging.exception('Unexpected exception in handler')
            msg_type = TMessageType.EXCEPTION
            result = TApplicationException(TApplicationException.INTERNAL_ERROR, 'Internal error')
        oprot.writeMessageBegin("startSession", msg_type, seqid)
        result.write(oprot)
        oprot.writeMessageEnd()
        oprot.trans.flush()

    def process_stopSession(self, seqid, iprot, oprot):
        args = stopSession_args()
        args.read(iprot)
        iprot.readMessageEnd()
        result = stopSession_result()
        try:
            self._handler.stopSession(args.sessionId)
            msg_type = TMessageType.REPLY
        except TTransport.TTransportException:
            raise
        except concrete.services.ttypes.ServicesException as ex:
            msg_type = TMessageType.REPLY
            result.ex = ex
        except TApplicationException as ex:
            logging.exception('TApplication exception in handler')
            msg_type = TMessageType.EXCEPTION
            result = ex
        except Exception:
            logging.exception('Unexpected exception in handler')
            msg_type = TMessageType.EXCEPTION
            result = TApplicationException(TApplicationException.INTERNAL_ERROR, 'Internal error')
        oprot.writeMessageBegin("stopSession", msg_type, seqid)
        result.write(oprot)
        oprot.writeMessageEnd()
        oprot.trans.flush()

    def process_getNextChunk(self, seqid, iprot, oprot):
        args = getNextChunk_args()
        args.read(iprot)
        iprot.readMessageEnd()
        result = getNextChunk_result()
        try:
            result.success = self._handler.getNextChunk(args.sessionId)
            msg_type = TMessageType.REPLY
        except TTransport.TTransportException:
            raise
        except concrete.services.ttypes.ServicesException as ex:
            msg_type = TMessageType.REPLY
            result.ex = ex
        except TApplicationException as ex:
            logging.exception('TApplication exception in handler')
            msg_type = TMessageType.EXCEPTION
            result = ex
        except Exception:
            logging.exception('Unexpected exception in handler')
            msg_type = TMessageType.EXCEPTION
            result = TApplicationException(TApplicationException.INTERNAL_ERROR, 'Internal error')
        oprot.writeMessageBegin("getNextChunk", msg_type, seqid)
        result.write(oprot)
        oprot.writeMessageEnd()
        oprot.trans.flush()

    def process_submitAnnotation(self, seqid, iprot, oprot):
        args = submitAnnotation_args()
        args.read(iprot)
        iprot.readMessageEnd()
        result = submitAnnotation_result()
        try:
            self._handler.submitAnnotation(args.sessionId, args.unitId, args.communication)
            msg_type = TMessageType.REPLY
        except TTransport.TTransportException:
            raise
        except concrete.services.ttypes.ServicesException as ex:
            msg_type = TMessageType.REPLY
            result.ex = ex
        except TApplicationException as ex:
            logging.exception('TApplication exception in handler')
            msg_type = TMessageType.EXCEPTION
            result = ex
        except Exception:
            logging.exception('Unexpected exception in handler')
            msg_type = TMessageType.EXCEPTION
            result = TApplicationException(TApplicationException.INTERNAL_ERROR, 'Internal error')
        oprot.writeMessageBegin("submitAnnotation", msg_type, seqid)
        result.write(oprot)
        oprot.writeMessageEnd()
        oprot.trans.flush()

# HELPER FUNCTIONS AND STRUCTURES


class registerSearchResult_args(object):
    """
    Attributes:
     - result
     - taskType

    """


    def __init__(self, result=None, taskType=None,):
        self.result = result
        self.taskType = taskType

    def read(self, iprot):
        if iprot._fast_decode is not None and isinstance(iprot.trans, TTransport.CReadableTransport) and self.thrift_spec is not None:
            iprot._fast_decode(self, iprot, [self.__class__, self.thrift_spec])
            return
        iprot.readStructBegin()
        while True:
            (fname, ftype, fid) = iprot.readFieldBegin()
            if ftype == TType.STOP:
                break
            if fid == 1:
                if ftype == TType.STRUCT:
                    self.result = concrete.search.ttypes.SearchResult()
                    self.result.read(iprot)
                else:
                    iprot.skip(ftype)
            elif fid == 2:
                if ftype == TType.I32:
                    self.taskType = iprot.readI32()
                else:
                    iprot.skip(ftype)
            else:
                iprot.skip(ftype)
            iprot.readFieldEnd()
        iprot.readStructEnd()

    def write(self, oprot):
        if oprot._fast_encode is not None and self.thrift_spec is not None:
            oprot.trans.write(oprot._fast_encode(self, [self.__class__, self.thrift_spec]))
            return
        oprot.writeStructBegin('registerSearchResult_args')
        if self.result is not None:
            oprot.writeFieldBegin('result', TType.STRUCT, 1)
            self.result.write(oprot)
            oprot.writeFieldEnd()
        if self.taskType is not None:
            oprot.writeFieldBegin('taskType', TType.I32, 2)
            oprot.writeI32(self.taskType)
            oprot.writeFieldEnd()
        oprot.writeFieldStop()
        oprot.writeStructEnd()

    def validate(self):
        return

    def __repr__(self):
        L = ['%s=%r' % (key, value)
             for key, value in self.__dict__.items()]
        return '%s(%s)' % (self.__class__.__name__, ', '.join(L))

    def __eq__(self, other):
        return isinstance(other, self.__class__) and self.__dict__ == other.__dict__

    def __ne__(self, other):
        return not (self == other)
all_structs.append(registerSearchResult_args)
registerSearchResult_args.thrift_spec = (
    None,  # 0
    (1, TType.STRUCT, 'result', [concrete.search.ttypes.SearchResult, None], None, ),  # 1
    (2, TType.I32, 'taskType', None, None, ),  # 2
)


class registerSearchResult_result(object):
    """
    Attributes:
     - ex

    """


    def __init__(self, ex=None,):
        self.ex = ex

    def read(self, iprot):
        if iprot._fast_decode is not None and isinstance(iprot.trans, TTransport.CReadableTransport) and self.thrift_spec is not None:
            iprot._fast_decode(self, iprot, [self.__class__, self.thrift_spec])
            return
        iprot.readStructBegin()
        while True:
            (fname, ftype, fid) = iprot.readFieldBegin()
            if ftype == TType.STOP:
                break
            if fid == 1:
                if ftype == TType.STRUCT:
                    self.ex = concrete.services.ttypes.ServicesException.read(iprot)
                else:
                    iprot.skip(ftype)
            else:
                iprot.skip(ftype)
            iprot.readFieldEnd()
        iprot.readStructEnd()

    def write(self, oprot):
        if oprot._fast_encode is not None and self.thrift_spec is not None:
            oprot.trans.write(oprot._fast_encode(self, [self.__class__, self.thrift_spec]))
            return
        oprot.writeStructBegin('registerSearchResult_result')
        if self.ex is not None:
            oprot.writeFieldBegin('ex', TType.STRUCT, 1)
            self.ex.write(oprot)
            oprot.writeFieldEnd()
        oprot.writeFieldStop()
        oprot.writeStructEnd()

    def validate(self):
        return

    def __repr__(self):
        L = ['%s=%r' % (key, value)
             for key, value in self.__dict__.items()]
        return '%s(%s)' % (self.__class__.__name__, ', '.join(L))

    def __eq__(self, other):
        return isinstance(other, self.__class__) and self.__dict__ == other.__dict__

    def __ne__(self, other):
        return not (self == other)
all_structs.append(registerSearchResult_result)
registerSearchResult_result.thrift_spec = (
    None,  # 0
    (1, TType.STRUCT, 'ex', [concrete.services.ttypes.ServicesException, None], None, ),  # 1
)


class getSearchResults_args(object):
    """
    Attributes:
     - taskType
     - limit

    """


    def __init__(self, taskType=None, limit=None,):
        self.taskType = taskType
        self.limit = limit

    def read(self, iprot):
        if iprot._fast_decode is not None and isinstance(iprot.trans, TTransport.CReadableTransport) and self.thrift_spec is not None:
            iprot._fast_decode(self, iprot, [self.__class__, self.thrift_spec])
            return
        iprot.readStructBegin()
        while True:
            (fname, ftype, fid) = iprot.readFieldBegin()
            if ftype == TType.STOP:
                break
            if fid == 1:
                if ftype == TType.I32:
                    self.taskType = iprot.readI32()
                else:
                    iprot.skip(ftype)
            elif fid == 2:
                if ftype == TType.I32:
                    self.limit = iprot.readI32()
                else:
                    iprot.skip(ftype)
            else:
                iprot.skip(ftype)
            iprot.readFieldEnd()
        iprot.readStructEnd()

    def write(self, oprot):
        if oprot._fast_encode is not None and self.thrift_spec is not None:
            oprot.trans.write(oprot._fast_encode(self, [self.__class__, self.thrift_spec]))
            return
        oprot.writeStructBegin('getSearchResults_args')
        if self.taskType is not None:
            oprot.writeFieldBegin('taskType', TType.I32, 1)
            oprot.writeI32(self.taskType)
            oprot.writeFieldEnd()
        if self.limit is not None:
            oprot.writeFieldBegin('limit', TType.I32, 2)
            oprot.writeI32(self.limit)
            oprot.writeFieldEnd()
        oprot.writeFieldStop()
        oprot.writeStructEnd()

    def validate(self):
        return

    def __repr__(self):
        L = ['%s=%r' % (key, value)
             for key, value in self.__dict__.items()]
        return '%s(%s)' % (self.__class__.__name__, ', '.join(L))

    def __eq__(self, other):
        return isinstance(other, self.__class__) and self.__dict__ == other.__dict__

    def __ne__(self, other):
        return not (self == other)
all_structs.append(getSearchResults_args)
getSearchResults_args.thrift_spec = (
    None,  # 0
    (1, TType.I32, 'taskType', None, None, ),  # 1
    (2, TType.I32, 'limit', None, None, ),  # 2
)


class getSearchResults_result(object):
    """
    Attributes:
     - success
     - ex

    """


    def __init__(self, success=None, ex=None,):
        self.success = success
        self.ex = ex

    def read(self, iprot):
        if iprot._fast_decode is not None and isinstance(iprot.trans, TTransport.CReadableTransport) and self.thrift_spec is not None:
            iprot._fast_decode(self, iprot, [self.__class__, self.thrift_spec])
            return
        iprot.readStructBegin()
        while True:
            (fname, ftype, fid) = iprot.readFieldBegin()
            if ftype == TType.STOP:
                break
            if fid == 0:
                if ftype == TType.LIST:
                    self.success = []
                    (_etype3, _size0) = iprot.readListBegin()
                    for _i4 in range(_size0):
                        _elem5 = concrete.search.ttypes.SearchResult()
                        _elem5.read(iprot)
                        self.success.append(_elem5)
                    iprot.readListEnd()
                else:
                    iprot.skip(ftype)
            elif fid == 1:
                if ftype == TType.STRUCT:
                    self.ex = concrete.services.ttypes.ServicesException.read(iprot)
                else:
                    iprot.skip(ftype)
            else:
                iprot.skip(ftype)
            iprot.readFieldEnd()
        iprot.readStructEnd()

    def write(self, oprot):
        if oprot._fast_encode is not None and self.thrift_spec is not None:
            oprot.trans.write(oprot._fast_encode(self, [self.__class__, self.thrift_spec]))
            return
        oprot.writeStructBegin('getSearchResults_result')
        if self.success is not None:
            oprot.writeFieldBegin('success', TType.LIST, 0)
            oprot.writeListBegin(TType.STRUCT, len(self.success))
            for iter6 in self.success:
                iter6.write(oprot)
            oprot.writeListEnd()
            oprot.writeFieldEnd()
        if self.ex is not None:
            oprot.writeFieldBegin('ex', TType.STRUCT, 1)
            self.ex.write(oprot)
            oprot.writeFieldEnd()
        oprot.writeFieldStop()
        oprot.writeStructEnd()

    def validate(self):
        return

    def __repr__(self):
        L = ['%s=%r' % (key, value)
             for key, value in self.__dict__.items()]
        return '%s(%s)' % (self.__class__.__name__, ', '.join(L))

    def __eq__(self, other):
        return isinstance(other, self.__class__) and self.__dict__ == other.__dict__

    def __ne__(self, other):
        return not (self == other)
all_structs.append(getSearchResults_result)
getSearchResults_result.thrift_spec = (
    (0, TType.LIST, 'success', (TType.STRUCT, [concrete.search.ttypes.SearchResult, None], False), None, ),  # 0
    (1, TType.STRUCT, 'ex', [concrete.services.ttypes.ServicesException, None], None, ),  # 1
)


class getSearchResultsByUser_args(object):
    """
    Attributes:
     - taskType
     - userId
     - limit

    """


    def __init__(self, taskType=None, userId=None, limit=None,):
        self.taskType = taskType
        self.userId = userId
        self.limit = limit

    def read(self, iprot):
        if iprot._fast_decode is not None and isinstance(iprot.trans, TTransport.CReadableTransport) and self.thrift_spec is not None:
            iprot._fast_decode(self, iprot, [self.__class__, self.thrift_spec])
            return
        iprot.readStructBegin()
        while True:
            (fname, ftype, fid) = iprot.readFieldBegin()
            if ftype == TType.STOP:
                break
            if fid == 1:
                if ftype == TType.I32:
                    self.taskType = iprot.readI32()
                else:
                    iprot.skip(ftype)
            elif fid == 2:
                if ftype == TType.STRING:
                    self.userId = iprot.readString().decode('utf-8', errors='replace') if sys.version_info[0] == 2 else iprot.readString()
                else:
                    iprot.skip(ftype)
            elif fid == 3:
                if ftype == TType.I32:
                    self.limit = iprot.readI32()
                else:
                    iprot.skip(ftype)
            else:
                iprot.skip(ftype)
            iprot.readFieldEnd()
        iprot.readStructEnd()

    def write(self, oprot):
        if oprot._fast_encode is not None and self.thrift_spec is not None:
            oprot.trans.write(oprot._fast_encode(self, [self.__class__, self.thrift_spec]))
            return
        oprot.writeStructBegin('getSearchResultsByUser_args')
        if self.taskType is not None:
            oprot.writeFieldBegin('taskType', TType.I32, 1)
            oprot.writeI32(self.taskType)
            oprot.writeFieldEnd()
        if self.userId is not None:
            oprot.writeFieldBegin('userId', TType.STRING, 2)
            oprot.writeString(self.userId.encode('utf-8') if sys.version_info[0] == 2 else self.userId)
            oprot.writeFieldEnd()
        if self.limit is not None:
            oprot.writeFieldBegin('limit', TType.I32, 3)
            oprot.writeI32(self.limit)
            oprot.writeFieldEnd()
        oprot.writeFieldStop()
        oprot.writeStructEnd()

    def validate(self):
        return

    def __repr__(self):
        L = ['%s=%r' % (key, value)
             for key, value in self.__dict__.items()]
        return '%s(%s)' % (self.__class__.__name__, ', '.join(L))

    def __eq__(self, other):
        return isinstance(other, self.__class__) and self.__dict__ == other.__dict__

    def __ne__(self, other):
        return not (self == other)
all_structs.append(getSearchResultsByUser_args)
getSearchResultsByUser_args.thrift_spec = (
    None,  # 0
    (1, TType.I32, 'taskType', None, None, ),  # 1
    (2, TType.STRING, 'userId', 'UTF8', None, ),  # 2
    (3, TType.I32, 'limit', None, None, ),  # 3
)


class getSearchResultsByUser_result(object):
    """
    Attributes:
     - success
     - ex

    """


    def __init__(self, success=None, ex=None,):
        self.success = success
        self.ex = ex

    def read(self, iprot):
        if iprot._fast_decode is not None and isinstance(iprot.trans, TTransport.CReadableTransport) and self.thrift_spec is not None:
            iprot._fast_decode(self, iprot, [self.__class__, self.thrift_spec])
            return
        iprot.readStructBegin()
        while True:
            (fname, ftype, fid) = iprot.readFieldBegin()
            if ftype == TType.STOP:
                break
            if fid == 0:
                if ftype == TType.LIST:
                    self.success = []
                    (_etype10, _size7) = iprot.readListBegin()
                    for _i11 in range(_size7):
                        _elem12 = concrete.search.ttypes.SearchResult()
                        _elem12.read(iprot)
                        self.success.append(_elem12)
                    iprot.readListEnd()
                else:
                    iprot.skip(ftype)
            elif fid == 1:
                if ftype == TType.STRUCT:
                    self.ex = concrete.services.ttypes.ServicesException.read(iprot)
                else:
                    iprot.skip(ftype)
            else:
                iprot.skip(ftype)
            iprot.readFieldEnd()
        iprot.readStructEnd()

    def write(self, oprot):
        if oprot._fast_encode is not None and self.thrift_spec is not None:
            oprot.trans.write(oprot._fast_encode(self, [self.__class__, self.thrift_spec]))
            return
        oprot.writeStructBegin('getSearchResultsByUser_result')
        if self.success is not None:
            oprot.writeFieldBegin('success', TType.LIST, 0)
            oprot.writeListBegin(TType.STRUCT, len(self.success))
            for iter13 in self.success:
                iter13.write(oprot)
            oprot.writeListEnd()
            oprot.writeFieldEnd()
        if self.ex is not None:
            oprot.writeFieldBegin('ex', TType.STRUCT, 1)
            self.ex.write(oprot)
            oprot.writeFieldEnd()
        oprot.writeFieldStop()
        oprot.writeStructEnd()

    def validate(self):
        return

    def __repr__(self):
        L = ['%s=%r' % (key, value)
             for key, value in self.__dict__.items()]
        return '%s(%s)' % (self.__class__.__name__, ', '.join(L))

    def __eq__(self, other):
        return isinstance(other, self.__class__) and self.__dict__ == other.__dict__

    def __ne__(self, other):
        return not (self == other)
all_structs.append(getSearchResultsByUser_result)
getSearchResultsByUser_result.thrift_spec = (
    (0, TType.LIST, 'success', (TType.STRUCT, [concrete.search.ttypes.SearchResult, None], False), None, ),  # 0
    (1, TType.STRUCT, 'ex', [concrete.services.ttypes.ServicesException, None], None, ),  # 1
)


class getLatestSearchResult_args(object):
    """
    Attributes:
     - userId

    """


    def __init__(self, userId=None,):
        self.userId = userId

    def read(self, iprot):
        if iprot._fast_decode is not None and isinstance(iprot.trans, TTransport.CReadableTransport) and self.thrift_spec is not None:
            iprot._fast_decode(self, iprot, [self.__class__, self.thrift_spec])
            return
        iprot.readStructBegin()
        while True:
            (fname, ftype, fid) = iprot.readFieldBegin()
            if ftype == TType.STOP:
                break
            if fid == 1:
                if ftype == TType.STRING:
                    self.userId = iprot.readString().decode('utf-8', errors='replace') if sys.version_info[0] == 2 else iprot.readString()
                else:
                    iprot.skip(ftype)
            else:
                iprot.skip(ftype)
            iprot.readFieldEnd()
        iprot.readStructEnd()

    def write(self, oprot):
        if oprot._fast_encode is not None and self.thrift_spec is not None:
            oprot.trans.write(oprot._fast_encode(self, [self.__class__, self.thrift_spec]))
            return
        oprot.writeStructBegin('getLatestSearchResult_args')
        if self.userId is not None:
            oprot.writeFieldBegin('userId', TType.STRING, 1)
            oprot.writeString(self.userId.encode('utf-8') if sys.version_info[0] == 2 else self.userId)
            oprot.writeFieldEnd()
        oprot.writeFieldStop()
        oprot.writeStructEnd()

    def validate(self):
        return

    def __repr__(self):
        L = ['%s=%r' % (key, value)
             for key, value in self.__dict__.items()]
        return '%s(%s)' % (self.__class__.__name__, ', '.join(L))

    def __eq__(self, other):
        return isinstance(other, self.__class__) and self.__dict__ == other.__dict__

    def __ne__(self, other):
        return not (self == other)
all_structs.append(getLatestSearchResult_args)
getLatestSearchResult_args.thrift_spec = (
    None,  # 0
    (1, TType.STRING, 'userId', 'UTF8', None, ),  # 1
)


class getLatestSearchResult_result(object):
    """
    Attributes:
     - success
     - ex

    """


    def __init__(self, success=None, ex=None,):
        self.success = success
        self.ex = ex

    def read(self, iprot):
        if iprot._fast_decode is not None and isinstance(iprot.trans, TTransport.CReadableTransport) and self.thrift_spec is not None:
            iprot._fast_decode(self, iprot, [self.__class__, self.thrift_spec])
            return
        iprot.readStructBegin()
        while True:
            (fname, ftype, fid) = iprot.readFieldBegin()
            if ftype == TType.STOP:
                break
            if fid == 0:
                if ftype == TType.STRUCT:
                    self.success = concrete.search.ttypes.SearchResult()
                    self.success.read(iprot)
                else:
                    iprot.skip(ftype)
            elif fid == 1:
                if ftype == TType.STRUCT:
                    self.ex = concrete.services.ttypes.ServicesException.read(iprot)
                else:
                    iprot.skip(ftype)
            else:
                iprot.skip(ftype)
            iprot.readFieldEnd()
        iprot.readStructEnd()

    def write(self, oprot):
        if oprot._fast_encode is not None and self.thrift_spec is not None:
            oprot.trans.write(oprot._fast_encode(self, [self.__class__, self.thrift_spec]))
            return
        oprot.writeStructBegin('getLatestSearchResult_result')
        if self.success is not None:
            oprot.writeFieldBegin('success', TType.STRUCT, 0)
            self.success.write(oprot)
            oprot.writeFieldEnd()
        if self.ex is not None:
            oprot.writeFieldBegin('ex', TType.STRUCT, 1)
            self.ex.write(oprot)
            oprot.writeFieldEnd()
        oprot.writeFieldStop()
        oprot.writeStructEnd()

    def validate(self):
        return

    def __repr__(self):
        L = ['%s=%r' % (key, value)
             for key, value in self.__dict__.items()]
        return '%s(%s)' % (self.__class__.__name__, ', '.join(L))

    def __eq__(self, other):
        return isinstance(other, self.__class__) and self.__dict__ == other.__dict__

    def __ne__(self, other):
        return not (self == other)
all_structs.append(getLatestSearchResult_result)
getLatestSearchResult_result.thrift_spec = (
    (0, TType.STRUCT, 'success', [concrete.search.ttypes.SearchResult, None], None, ),  # 0
    (1, TType.STRUCT, 'ex', [concrete.services.ttypes.ServicesException, None], None, ),  # 1
)


class getSearchResult_args(object):
    """
    Attributes:
     - searchResultId

    """


    def __init__(self, searchResultId=None,):
        self.searchResultId = searchResultId

    def read(self, iprot):
        if iprot._fast_decode is not None and isinstance(iprot.trans, TTransport.CReadableTransport) and self.thrift_spec is not None:
            iprot._fast_decode(self, iprot, [self.__class__, self.thrift_spec])
            return
        iprot.readStructBegin()
        while True:
            (fname, ftype, fid) = iprot.readFieldBegin()
            if ftype == TType.STOP:
                break
            if fid == 1:
                if ftype == TType.STRUCT:
                    self.searchResultId = concrete.uuid.ttypes.UUID()
                    self.searchResultId.read(iprot)
                else:
                    iprot.skip(ftype)
            else:
                iprot.skip(ftype)
            iprot.readFieldEnd()
        iprot.readStructEnd()

    def write(self, oprot):
        if oprot._fast_encode is not None and self.thrift_spec is not None:
            oprot.trans.write(oprot._fast_encode(self, [self.__class__, self.thrift_spec]))
            return
        oprot.writeStructBegin('getSearchResult_args')
        if self.searchResultId is not None:
            oprot.writeFieldBegin('searchResultId', TType.STRUCT, 1)
            self.searchResultId.write(oprot)
            oprot.writeFieldEnd()
        oprot.writeFieldStop()
        oprot.writeStructEnd()

    def validate(self):
        return

    def __repr__(self):
        L = ['%s=%r' % (key, value)
             for key, value in self.__dict__.items()]
        return '%s(%s)' % (self.__class__.__name__, ', '.join(L))

    def __eq__(self, other):
        return isinstance(other, self.__class__) and self.__dict__ == other.__dict__

    def __ne__(self, other):
        return not (self == other)
all_structs.append(getSearchResult_args)
getSearchResult_args.thrift_spec = (
    None,  # 0
    (1, TType.STRUCT, 'searchResultId', [concrete.uuid.ttypes.UUID, None], None, ),  # 1
)


class getSearchResult_result(object):
    """
    Attributes:
     - success
     - ex

    """


    def __init__(self, success=None, ex=None,):
        self.success = success
        self.ex = ex

    def read(self, iprot):
        if iprot._fast_decode is not None and isinstance(iprot.trans, TTransport.CReadableTransport) and self.thrift_spec is not None:
            iprot._fast_decode(self, iprot, [self.__class__, self.thrift_spec])
            return
        iprot.readStructBegin()
        while True:
            (fname, ftype, fid) = iprot.readFieldBegin()
            if ftype == TType.STOP:
                break
            if fid == 0:
                if ftype == TType.STRUCT:
                    self.success = concrete.search.ttypes.SearchResult()
                    self.success.read(iprot)
                else:
                    iprot.skip(ftype)
            elif fid == 1:
                if ftype == TType.STRUCT:
                    self.ex = concrete.services.ttypes.ServicesException.read(iprot)
                else:
                    iprot.skip(ftype)
            else:
                iprot.skip(ftype)
            iprot.readFieldEnd()
        iprot.readStructEnd()

    def write(self, oprot):
        if oprot._fast_encode is not None and self.thrift_spec is not None:
            oprot.trans.write(oprot._fast_encode(self, [self.__class__, self.thrift_spec]))
            return
        oprot.writeStructBegin('getSearchResult_result')
        if self.success is not None:
            oprot.writeFieldBegin('success', TType.STRUCT, 0)
            self.success.write(oprot)
            oprot.writeFieldEnd()
        if self.ex is not None:
            oprot.writeFieldBegin('ex', TType.STRUCT, 1)
            self.ex.write(oprot)
            oprot.writeFieldEnd()
        oprot.writeFieldStop()
        oprot.writeStructEnd()

    def validate(self):
        return

    def __repr__(self):
        L = ['%s=%r' % (key, value)
             for key, value in self.__dict__.items()]
        return '%s(%s)' % (self.__class__.__name__, ', '.join(L))

    def __eq__(self, other):
        return isinstance(other, self.__class__) and self.__dict__ == other.__dict__

    def __ne__(self, other):
        return not (self == other)
all_structs.append(getSearchResult_result)
getSearchResult_result.thrift_spec = (
    (0, TType.STRUCT, 'success', [concrete.search.ttypes.SearchResult, None], None, ),  # 0
    (1, TType.STRUCT, 'ex', [concrete.services.ttypes.ServicesException, None], None, ),  # 1
)


class startSession_args(object):
    """
    Attributes:
     - searchResultId
     - taskType

    """


    def __init__(self, searchResultId=None, taskType=None,):
        self.searchResultId = searchResultId
        self.taskType = taskType

    def read(self, iprot):
        if iprot._fast_decode is not None and isinstance(iprot.trans, TTransport.CReadableTransport) and self.thrift_spec is not None:
            iprot._fast_decode(self, iprot, [self.__class__, self.thrift_spec])
            return
        iprot.readStructBegin()
        while True:
            (fname, ftype, fid) = iprot.readFieldBegin()
            if ftype == TType.STOP:
                break
            if fid == 1:
                if ftype == TType.STRUCT:
                    self.searchResultId = concrete.uuid.ttypes.UUID()
                    self.searchResultId.read(iprot)
                else:
                    iprot.skip(ftype)
            elif fid == 2:
                if ftype == TType.I32:
                    self.taskType = iprot.readI32()
                else:
                    iprot.skip(ftype)
            else:
                iprot.skip(ftype)
            iprot.readFieldEnd()
        iprot.readStructEnd()

    def write(self, oprot):
        if oprot._fast_encode is not None and self.thrift_spec is not None:
            oprot.trans.write(oprot._fast_encode(self, [self.__class__, self.thrift_spec]))
            return
        oprot.writeStructBegin('startSession_args')
        if self.searchResultId is not None:
            oprot.writeFieldBegin('searchResultId', TType.STRUCT, 1)
            self.searchResultId.write(oprot)
            oprot.writeFieldEnd()
        if self.taskType is not None:
            oprot.writeFieldBegin('taskType', TType.I32, 2)
            oprot.writeI32(self.taskType)
            oprot.writeFieldEnd()
        oprot.writeFieldStop()
        oprot.writeStructEnd()

    def validate(self):
        return

    def __repr__(self):
        L = ['%s=%r' % (key, value)
             for key, value in self.__dict__.items()]
        return '%s(%s)' % (self.__class__.__name__, ', '.join(L))

    def __eq__(self, other):
        return isinstance(other, self.__class__) and self.__dict__ == other.__dict__

    def __ne__(self, other):
        return not (self == other)
all_structs.append(startSession_args)
startSession_args.thrift_spec = (
    None,  # 0
    (1, TType.STRUCT, 'searchResultId', [concrete.uuid.ttypes.UUID, None], None, ),  # 1
    (2, TType.I32, 'taskType', None, None, ),  # 2
)


class startSession_result(object):
    """
    Attributes:
     - success
     - ex

    """


    def __init__(self, success=None, ex=None,):
        self.success = success
        self.ex = ex

    def read(self, iprot):
        if iprot._fast_decode is not None and isinstance(iprot.trans, TTransport.CReadableTransport) and self.thrift_spec is not None:
            iprot._fast_decode(self, iprot, [self.__class__, self.thrift_spec])
            return
        iprot.readStructBegin()
        while True:
            (fname, ftype, fid) = iprot.readFieldBegin()
            if ftype == TType.STOP:
                break
            if fid == 0:
                if ftype == TType.STRUCT:
                    self.success = concrete.uuid.ttypes.UUID()
                    self.success.read(iprot)
                else:
                    iprot.skip(ftype)
            elif fid == 1:
                if ftype == TType.STRUCT:
                    self.ex = concrete.services.ttypes.ServicesException.read(iprot)
                else:
                    iprot.skip(ftype)
            else:
                iprot.skip(ftype)
            iprot.readFieldEnd()
        iprot.readStructEnd()

    def write(self, oprot):
        if oprot._fast_encode is not None and self.thrift_spec is not None:
            oprot.trans.write(oprot._fast_encode(self, [self.__class__, self.thrift_spec]))
            return
        oprot.writeStructBegin('startSession_result')
        if self.success is not None:
            oprot.writeFieldBegin('success', TType.STRUCT, 0)
            self.success.write(oprot)
            oprot.writeFieldEnd()
        if self.ex is not None:
            oprot.writeFieldBegin('ex', TType.STRUCT, 1)
            self.ex.write(oprot)
            oprot.writeFieldEnd()
        oprot.writeFieldStop()
        oprot.writeStructEnd()

    def validate(self):
        return

    def __repr__(self):
        L = ['%s=%r' % (key, value)
             for key, value in self.__dict__.items()]
        return '%s(%s)' % (self.__class__.__name__, ', '.join(L))

    def __eq__(self, other):
        return isinstance(other, self.__class__) and self.__dict__ == other.__dict__

    def __ne__(self, other):
        return not (self == other)
all_structs.append(startSession_result)
startSession_result.thrift_spec = (
    (0, TType.STRUCT, 'success', [concrete.uuid.ttypes.UUID, None], None, ),  # 0
    (1, TType.STRUCT, 'ex', [concrete.services.ttypes.ServicesException, None], None, ),  # 1
)


class stopSession_args(object):
    """
    Attributes:
     - sessionId

    """


    def __init__(self, sessionId=None,):
        self.sessionId = sessionId

    def read(self, iprot):
        if iprot._fast_decode is not None and isinstance(iprot.trans, TTransport.CReadableTransport) and self.thrift_spec is not None:
            iprot._fast_decode(self, iprot, [self.__class__, self.thrift_spec])
            return
        iprot.readStructBegin()
        while True:
            (fname, ftype, fid) = iprot.readFieldBegin()
            if ftype == TType.STOP:
                break
            if fid == 1:
                if ftype == TType.STRUCT:
                    self.sessionId = concrete.uuid.ttypes.UUID()
                    self.sessionId.read(iprot)
                else:
                    iprot.skip(ftype)
            else:
                iprot.skip(ftype)
            iprot.readFieldEnd()
        iprot.readStructEnd()

    def write(self, oprot):
        if oprot._fast_encode is not None and self.thrift_spec is not None:
            oprot.trans.write(oprot._fast_encode(self, [self.__class__, self.thrift_spec]))
            return
        oprot.writeStructBegin('stopSession_args')
        if self.sessionId is not None:
            oprot.writeFieldBegin('sessionId', TType.STRUCT, 1)
            self.sessionId.write(oprot)
            oprot.writeFieldEnd()
        oprot.writeFieldStop()
        oprot.writeStructEnd()

    def validate(self):
        return

    def __repr__(self):
        L = ['%s=%r' % (key, value)
             for key, value in self.__dict__.items()]
        return '%s(%s)' % (self.__class__.__name__, ', '.join(L))

    def __eq__(self, other):
        return isinstance(other, self.__class__) and self.__dict__ == other.__dict__

    def __ne__(self, other):
        return not (self == other)
all_structs.append(stopSession_args)
stopSession_args.thrift_spec = (
    None,  # 0
    (1, TType.STRUCT, 'sessionId', [concrete.uuid.ttypes.UUID, None], None, ),  # 1
)


class stopSession_result(object):
    """
    Attributes:
     - ex

    """


    def __init__(self, ex=None,):
        self.ex = ex

    def read(self, iprot):
        if iprot._fast_decode is not None and isinstance(iprot.trans, TTransport.CReadableTransport) and self.thrift_spec is not None:
            iprot._fast_decode(self, iprot, [self.__class__, self.thrift_spec])
            return
        iprot.readStructBegin()
        while True:
            (fname, ftype, fid) = iprot.readFieldBegin()
            if ftype == TType.STOP:
                break
            if fid == 1:
                if ftype == TType.STRUCT:
                    self.ex = concrete.services.ttypes.ServicesException.read(iprot)
                else:
                    iprot.skip(ftype)
            else:
                iprot.skip(ftype)
            iprot.readFieldEnd()
        iprot.readStructEnd()

    def write(self, oprot):
        if oprot._fast_encode is not None and self.thrift_spec is not None:
            oprot.trans.write(oprot._fast_encode(self, [self.__class__, self.thrift_spec]))
            return
        oprot.writeStructBegin('stopSession_result')
        if self.ex is not None:
            oprot.writeFieldBegin('ex', TType.STRUCT, 1)
            self.ex.write(oprot)
            oprot.writeFieldEnd()
        oprot.writeFieldStop()
        oprot.writeStructEnd()

    def validate(self):
        return

    def __repr__(self):
        L = ['%s=%r' % (key, value)
             for key, value in self.__dict__.items()]
        return '%s(%s)' % (self.__class__.__name__, ', '.join(L))

    def __eq__(self, other):
        return isinstance(other, self.__class__) and self.__dict__ == other.__dict__

    def __ne__(self, other):
        return not (self == other)
all_structs.append(stopSession_result)
stopSession_result.thrift_spec = (
    None,  # 0
    (1, TType.STRUCT, 'ex', [concrete.services.ttypes.ServicesException, None], None, ),  # 1
)


class getNextChunk_args(object):
    """
    Attributes:
     - sessionId

    """


    def __init__(self, sessionId=None,):
        self.sessionId = sessionId

    def read(self, iprot):
        if iprot._fast_decode is not None and isinstance(iprot.trans, TTransport.CReadableTransport) and self.thrift_spec is not None:
            iprot._fast_decode(self, iprot, [self.__class__, self.thrift_spec])
            return
        iprot.readStructBegin()
        while True:
            (fname, ftype, fid) = iprot.readFieldBegin()
            if ftype == TType.STOP:
                break
            if fid == 1:
                if ftype == TType.STRUCT:
                    self.sessionId = concrete.uuid.ttypes.UUID()
                    self.sessionId.read(iprot)
                else:
                    iprot.skip(ftype)
            else:
                iprot.skip(ftype)
            iprot.readFieldEnd()
        iprot.readStructEnd()

    def write(self, oprot):
        if oprot._fast_encode is not None and self.thrift_spec is not None:
            oprot.trans.write(oprot._fast_encode(self, [self.__class__, self.thrift_spec]))
            return
        oprot.writeStructBegin('getNextChunk_args')
        if self.sessionId is not None:
            oprot.writeFieldBegin('sessionId', TType.STRUCT, 1)
            self.sessionId.write(oprot)
            oprot.writeFieldEnd()
        oprot.writeFieldStop()
        oprot.writeStructEnd()

    def validate(self):
        return

    def __repr__(self):
        L = ['%s=%r' % (key, value)
             for key, value in self.__dict__.items()]
        return '%s(%s)' % (self.__class__.__name__, ', '.join(L))

    def __eq__(self, other):
        return isinstance(other, self.__class__) and self.__dict__ == other.__dict__

    def __ne__(self, other):
        return not (self == other)
all_structs.append(getNextChunk_args)
getNextChunk_args.thrift_spec = (
    None,  # 0
    (1, TType.STRUCT, 'sessionId', [concrete.uuid.ttypes.UUID, None], None, ),  # 1
)


class getNextChunk_result(object):
    """
    Attributes:
     - success
     - ex

    """


    def __init__(self, success=None, ex=None,):
        self.success = success
        self.ex = ex

    def read(self, iprot):
        if iprot._fast_decode is not None and isinstance(iprot.trans, TTransport.CReadableTransport) and self.thrift_spec is not None:
            iprot._fast_decode(self, iprot, [self.__class__, self.thrift_spec])
            return
        iprot.readStructBegin()
        while True:
            (fname, ftype, fid) = iprot.readFieldBegin()
            if ftype == TType.STOP:
                break
            if fid == 0:
                if ftype == TType.LIST:
                    self.success = []
                    (_etype17, _size14) = iprot.readListBegin()
                    for _i18 in range(_size14):
                        _elem19 = concrete.services.ttypes.AnnotationUnitIdentifier()
                        _elem19.read(iprot)
                        self.success.append(_elem19)
                    iprot.readListEnd()
                else:
                    iprot.skip(ftype)
            elif fid == 1:
                if ftype == TType.STRUCT:
                    self.ex = concrete.services.ttypes.ServicesException.read(iprot)
                else:
                    iprot.skip(ftype)
            else:
                iprot.skip(ftype)
            iprot.readFieldEnd()
        iprot.readStructEnd()

    def write(self, oprot):
        if oprot._fast_encode is not None and self.thrift_spec is not None:
            oprot.trans.write(oprot._fast_encode(self, [self.__class__, self.thrift_spec]))
            return
        oprot.writeStructBegin('getNextChunk_result')
        if self.success is not None:
            oprot.writeFieldBegin('success', TType.LIST, 0)
            oprot.writeListBegin(TType.STRUCT, len(self.success))
            for iter20 in self.success:
                iter20.write(oprot)
            oprot.writeListEnd()
            oprot.writeFieldEnd()
        if self.ex is not None:
            oprot.writeFieldBegin('ex', TType.STRUCT, 1)
            self.ex.write(oprot)
            oprot.writeFieldEnd()
        oprot.writeFieldStop()
        oprot.writeStructEnd()

    def validate(self):
        return

    def __repr__(self):
        L = ['%s=%r' % (key, value)
             for key, value in self.__dict__.items()]
        return '%s(%s)' % (self.__class__.__name__, ', '.join(L))

    def __eq__(self, other):
        return isinstance(other, self.__class__) and self.__dict__ == other.__dict__

    def __ne__(self, other):
        return not (self == other)
all_structs.append(getNextChunk_result)
getNextChunk_result.thrift_spec = (
    (0, TType.LIST, 'success', (TType.STRUCT, [concrete.services.ttypes.AnnotationUnitIdentifier, None], False), None, ),  # 0
    (1, TType.STRUCT, 'ex', [concrete.services.ttypes.ServicesException, None], None, ),  # 1
)


class submitAnnotation_args(object):
    """
    Attributes:
     - sessionId
     - unitId
     - communication

    """


    def __init__(self, sessionId=None, unitId=None, communication=None,):
        self.sessionId = sessionId
        self.unitId = unitId
        self.communication = communication

    def read(self, iprot):
        if iprot._fast_decode is not None and isinstance(iprot.trans, TTransport.CReadableTransport) and self.thrift_spec is not None:
            iprot._fast_decode(self, iprot, [self.__class__, self.thrift_spec])
            return
        iprot.readStructBegin()
        while True:
            (fname, ftype, fid) = iprot.readFieldBegin()
            if ftype == TType.STOP:
                break
            if fid == 1:
                if ftype == TType.STRUCT:
                    self.sessionId = concrete.uuid.ttypes.UUID()
                    self.sessionId.read(iprot)
                else:
                    iprot.skip(ftype)
            elif fid == 2:
                if ftype == TType.STRUCT:
                    self.unitId = concrete.services.ttypes.AnnotationUnitIdentifier()
                    self.unitId.read(iprot)
                else:
                    iprot.skip(ftype)
            elif fid == 3:
                if ftype == TType.STRUCT:
                    self.communication = concrete.communication.ttypes.Communication()
                    self.communication.read(iprot)
                else:
                    iprot.skip(ftype)
            else:
                iprot.skip(ftype)
            iprot.readFieldEnd()
        iprot.readStructEnd()

    def write(self, oprot):
        if oprot._fast_encode is not None and self.thrift_spec is not None:
            oprot.trans.write(oprot._fast_encode(self, [self.__class__, self.thrift_spec]))
            return
        oprot.writeStructBegin('submitAnnotation_args')
        if self.sessionId is not None:
            oprot.writeFieldBegin('sessionId', TType.STRUCT, 1)
            self.sessionId.write(oprot)
            oprot.writeFieldEnd()
        if self.unitId is not None:
            oprot.writeFieldBegin('unitId', TType.STRUCT, 2)
            self.unitId.write(oprot)
            oprot.writeFieldEnd()
        if self.communication is not None:
            oprot.writeFieldBegin('communication', TType.STRUCT, 3)
            self.communication.write(oprot)
            oprot.writeFieldEnd()
        oprot.writeFieldStop()
        oprot.writeStructEnd()

    def validate(self):
        return

    def __repr__(self):
        L = ['%s=%r' % (key, value)
             for key, value in self.__dict__.items()]
        return '%s(%s)' % (self.__class__.__name__, ', '.join(L))

    def __eq__(self, other):
        return isinstance(other, self.__class__) and self.__dict__ == other.__dict__

    def __ne__(self, other):
        return not (self == other)
all_structs.append(submitAnnotation_args)
submitAnnotation_args.thrift_spec = (
    None,  # 0
    (1, TType.STRUCT, 'sessionId', [concrete.uuid.ttypes.UUID, None], None, ),  # 1
    (2, TType.STRUCT, 'unitId', [concrete.services.ttypes.AnnotationUnitIdentifier, None], None, ),  # 2
    (3, TType.STRUCT, 'communication', [concrete.communication.ttypes.Communication, None], None, ),  # 3
)


class submitAnnotation_result(object):
    """
    Attributes:
     - ex

    """


    def __init__(self, ex=None,):
        self.ex = ex

    def read(self, iprot):
        if iprot._fast_decode is not None and isinstance(iprot.trans, TTransport.CReadableTransport) and self.thrift_spec is not None:
            iprot._fast_decode(self, iprot, [self.__class__, self.thrift_spec])
            return
        iprot.readStructBegin()
        while True:
            (fname, ftype, fid) = iprot.readFieldBegin()
            if ftype == TType.STOP:
                break
            if fid == 1:
                if ftype == TType.STRUCT:
                    self.ex = concrete.services.ttypes.ServicesException.read(iprot)
                else:
                    iprot.skip(ftype)
            else:
                iprot.skip(ftype)
            iprot.readFieldEnd()
        iprot.readStructEnd()

    def write(self, oprot):
        if oprot._fast_encode is not None and self.thrift_spec is not None:
            oprot.trans.write(oprot._fast_encode(self, [self.__class__, self.thrift_spec]))
            return
        oprot.writeStructBegin('submitAnnotation_result')
        if self.ex is not None:
            oprot.writeFieldBegin('ex', TType.STRUCT, 1)
            self.ex.write(oprot)
            oprot.writeFieldEnd()
        oprot.writeFieldStop()
        oprot.writeStructEnd()

    def validate(self):
        return

    def __repr__(self):
        L = ['%s=%r' % (key, value)
             for key, value in self.__dict__.items()]
        return '%s(%s)' % (self.__class__.__name__, ', '.join(L))

    def __eq__(self, other):
        return isinstance(other, self.__class__) and self.__dict__ == other.__dict__

    def __ne__(self, other):
        return not (self == other)
all_structs.append(submitAnnotation_result)
submitAnnotation_result.thrift_spec = (
    None,  # 0
    (1, TType.STRUCT, 'ex', [concrete.services.ttypes.ServicesException, None], None, ),  # 1
)
fix_spec(all_structs)
del all_structs



from __future__ import unicode_literals
import os
import sys


def import_ok(module_name):
    try:
        module = __import__(module_name)
        sys.stdout.write('import succeeded: %s -> %s\n' %
                         (module_name, module))
        return True
    except Exception as ex:
        sys.stderr.write('import failed: %s: %s\n' %
                         (module_name, ex.message))
        return False


def test_imports():
    root = 'concrete'
    for (parent_path, dir_entries, file_entries) in os.walk(root):
        for basename in dir_entries:
            if not basename.endswith('__pycache__'):
                dir_path = os.path.join(parent_path, basename)
                module_name = dir_path.replace(os.sep, '.')
                assert import_ok(module_name)
        for basename in file_entries:
            if basename.endswith('.py') and basename != '__init__.py':
                file_path = os.path.join(parent_path, basename)
                module_name = file_path[:-len('.py')].replace(os.sep, '.')
                assert import_ok(module_name)



from __future__ import unicode_literals
from datetime import datetime
import json
import logging


EPOCH = datetime.utcfromtimestamp(0)


class ZeroAnnotationsError(Exception):
    '''
    Exception representing zero annotations present in a concrete object
    when one (or more) is expected.
    '''
    def __init__(self, *args, **kwargs):
        Exception.__init__(self, *args, **kwargs)


class MultipleAnnotationsError(Exception):
    '''
    Exception representing more than one annotations present in a
    concrete object when one (or zero) is expected.
    '''
    def __init__(self, *args, **kwargs):
        Exception.__init__(self, *args, **kwargs)


def datetime_to_timestamp(dt):
    '''
    Given time-zone--unaware datetime object representing date and time
    in UTC, return corresponding Concrete timestamp.

    Args:
        dt (datetime): time-zone--unaware datetime object representing
            date and time (in UTC) to convert

    Returns:
        concrete timestamp representing datetime dt

    Source:
    http://stackoverflow.com/questions/6999726/how-can-i-convert-a-datetime-object-to-milliseconds-since-epoch-unix-time-in-p
    '''
    return int((dt - EPOCH).total_seconds())


def timestamp_to_datetime(timestamp):
    '''
    Given Concrete timestamp, return corresponding time-zone--unaware
    datetime object representing date and time in UTC.

    Args:
        timestamp(int): Concrete timestamp (integer representing
            seconds since the epoch in UTC) representing date and time
            to convert

    Returns:
        datetime representing timestamp dt

    Source:
    https://stackoverflow.com/questions/3694487/initialize-a-datetime-object-with-seconds-since-epoch
    '''
    return datetime.utcfromtimestamp(timestamp)


def now_timestamp():
    '''
    Return timestamp representing the current time.

    Returns:
        concrete timestamp representing the current time
    '''
    return datetime_to_timestamp(datetime.now())


def get_index_of_tool(lst_of_conc, tool):
    """Return the index of the object in the provided list
    whose tool name matches tool.

    If tool is None, return the first valid index into `lst_of_conc`.

    This returns -1 if:
      * `lst_of_conc` is None, or
      * `lst_of_conc` has no entries, or
      * no object in `lst_of_conc` matches `tool`.

    Args:
        lst_of_conc (list): list of Concrete objects, each of which
            has a `.metadata` field.
        tool (str): A tool name to match.
    """
    idx = -1
    if lst_of_conc is not None and len(lst_of_conc) > 0:
        if tool is not None:
            for (cidx, obj) in enumerate(lst_of_conc):
                if obj.metadata.tool == tool:
                    idx = cidx
                    break
        else:
            idx = 0
    return idx


def get_annotation_field(annotation, field):
    '''
    Return requested field of annotation metadata.

    Args:
        annotation (object): object containing a `metadata` field of
            type :class:`..metadata.ttypes.AnnotationMetadata`.
        field (str): name of metadata field: kBest, timestamp, or tool.

    Returns:
        value of requested field in annotation metadata.

    Raises:
        ValueError: on unknown field name
    '''
    if field == 'kBest':
        return annotation.metadata.kBest
    elif field == 'timestamp':
        return annotation.metadata.timestamp
    elif field == 'tool':
        return annotation.metadata.tool
    else:
        raise ValueError('unrecognized field {}'.format(field))


def filter_annotations(annotations,
                       filter_fields=None,
                       sort_field=None,
                       sort_reverse=False,
                       action_if_multiple='pass',
                       action_if_zero='pass'):
    '''
    Return filtered and/or re-ordered list of annotations, that is,
    objects containing a `metadata` field of type AnnotationMetadata.
    The default behavior is to do no filtering (or re-ordering),
    returning an exact copy of annotations.

    Args:
        annotations (list): original list of annotations (objects
            containing a `metadata` field of type
            :class:`..metadata.ttypes.AnnotationMetadata`).
            This list is not modified.
        filter_fields (dict): dict of fields and their desired values
            by which to filter annotations (keep annotations whose
            field `FIELD` equals `VALUE` for all `FIELD:
            VALUE`) entries).  Default: keep all annotations.
            See :func:`get_annotation_field` for valid fields.
        sort_field (str): field by which to re-order annotations.
            Default: do not re-order annotations.
        sort_reverse (bool): True to reverse order of annotations
            (after sorting, if any).
        action_if_multiple (str): action to take if, after filtering,
            there is more than one annotation left.  'pass' to
            return all filtered and re-ordered annotations, 'raise' to
            raise an exception of type `MultipleAnnotationsError`,
            'first' to return a list containing the first annotation
            after filtering and re-ordering, or 'last' to return a list
            containing the last annotation after filtering and
            re-ordering.
        action_if_zero (str): action to take if, after filtering, there
            are no annotations left.  'pass' to return an empty list,
            'raise' to raise an exception of type
            `ZeroAnnotationsError`.

    Returns:
        filtered and/or re-ordered list of annotations

    Raises:
        ValueError: if the value of action_if_multiple or action_if_zero
            is not recognized
        MultipleAnnotationsError: if the value of action_if_multiple is
            'raise' and there are multiple annotations passing the
            filter
        ZeroAnnotationsError: if the value of action_if_zero is
            'raise' and there are no annotations passing the filter
    '''
    annotations = list(annotations)

    if filter_fields:
        annotations = [
            a for a in annotations
            if all(
                get_annotation_field(a, field) == value
                for (field, value) in filter_fields.items()
            )
        ]

    if sort_field:
        annotations = sorted(
            annotations,
            key=lambda a: get_annotation_field(a, sort_field))

    if sort_reverse:
        annotations = annotations[::-1]

    if len(annotations) == 0:
        if action_if_zero == 'raise':
            raise ZeroAnnotationsError()
        elif action_if_zero == 'pass':
            pass
        else:
            raise ValueError('unknown action_if_zero value {}'.format(
                action_if_zero))
    elif len(annotations) > 1:
        if action_if_multiple == 'raise':
            raise MultipleAnnotationsError()
        elif action_if_multiple == 'pass':
            pass
        elif action_if_multiple == 'first':
            annotations = [annotations[0]]
        elif action_if_multiple == 'last':
            annotations = [annotations[-1]]
        else:
            raise ValueError('unknown action_if_multiple value {}'.format(
                action_if_multiple))

    return annotations


def filter_annotations_json(annotations, kwargs_json):
    '''
    Call :func:`filter_annotations` on `annotations`, sending it
    keyword arguments from the JSON-encoded dictionary
    `kwargs_json`.

    Args:
        annotations (list): original list of annotations (objects
            containing a `metadata` field of type
            :class:`..metadata.ttypes.AnnotationMetadata`).
            This list is not modified.
        kwargs_json (str): JSON-encoded dictionary of keyword
            arguments to be passed to :func:`filter_annotations`.

    Returns:
        `annotations` filtered by :func:`filter_annotations` according
        to provided JSON-encoded keyword arguments.

    Raises:
        ValueError: if the value of 'action_if_multiple' or
            'action_if_zero' is not recognized
        MultipleAnnotationsError: if the value of 'action_if_multiple'
            is 'raise' and there are multiple annotations passing the
            filter
        ZeroAnnotationsError: if the value of 'action_if_zero' is
            'raise' and there are no annotations passing the filter
    '''

    return filter_annotations(annotations, **json.loads(kwargs_json))


def filter_unnone(annotation_filter):
    '''
    If annotation_filter is None, return no-op filter.

    Args:
        annotation_filter (func): function that takes a list of
            annotations and returns a filtered (and/or re-ordered)
            list of annotations

    Returns:
        function that takes a list of annotations and returns a
        filtered (and/or re-ordered) list of annotations.
    '''
    if annotation_filter is None:
        return lambda annotations: annotations
    else:
        return annotation_filter


def tool_to_filter(tool, explicit_filter=None):
    '''
    Given tool name (deprecated way to filter annotations) or None,
    and an explicit annotation filter function or None, return an
    annotation filter function representing whichever is not None
    (and raise ValueError if both are not None).

    Args:
        tool (str): name of tool to filter by, or None
        explicit_filter (func): function taking a list of annotations
            as input and returning a sub-list (possibly re-ordered)
            as output, or None

    Returns:
        Function taking a list of annotations as input and either
        applying `explicit_filter` to them and returning its output
        or filtering them by tool `tool` and returning that filtered
        list.  If both `tool` and `explicit_filter` are not None,
        raise ValueError.

    Raises:
        ValueError: if both `tool` and `explicit_filter` are not None
    '''
    if tool is None:
        return explicit_filter
    else:
        logging.warning('tool is deprecated; please pass filter instead')
        if explicit_filter is None:
            return lambda annotations: filter_annotations(
                annotations,
                filter_fields=dict(tool=tool))
        else:
            raise ValueError('tool and filter cannot be both be specified')



#!/usr/bin/env python

from __future__ import print_function
from __future__ import unicode_literals
from argparse import ArgumentParser, ArgumentDefaultsHelpFormatter
import logging
import sys

from concrete.search.ttypes import SearchQuery, SearchType
from concrete.util import set_stdout_encoding
from concrete.util.search_wrapper import SearchClientWrapper
import concrete.version

try:
    raw_input
except NameError:
    raw_input = input


def execute_search_query(search_client, terms, k):
    logging.debug("executing query '{}'".format(u' '.join(terms)))
    query = SearchQuery(type=SearchType.COMMUNICATIONS, terms=terms, k=k)
    result = search_client.search(query)
    return [
        (item.communicationId, item.score)
        for item in result.searchResultItems
    ]


def unicode_arg(s):
    """Convert argparse argument to Unicode string

    On Python 3, no conversion is necessary
    On Python 2, we convert bytestring to unicode using file system encoding

    See:
      https://codereview.stackexchange.com/questions/124434/get-argument-as-unicode-string-from-argparse-in-python-2-and-3
    """
    if sys.version_info >= (3, 0):
        return s
    else:
        return s.decode(sys.getfilesystemencoding())


def main():
    set_stdout_encoding()

    parser = ArgumentParser(
        formatter_class=ArgumentDefaultsHelpFormatter,
        description='Concrete Search client (allows specifying query on '
                    'command line, interactively on the terminal, or '
                    'in batch from standard input).'
    )
    parser.add_argument('--host', default='localhost',
                        help='Hostname of Search service')
    parser.add_argument('--port', type=int, default=9090,
                        help='Port of Search service')
    parser.add_argument('--user-id', type=str,
                        help='user id to send to search service')
    parser.add_argument('-k', '--k', type=int, default=10,
                        help='Maximum number of search results to return per query')
    parser.add_argument('-i', '--interactive', action='store_true',
                        help='Start interactive client (read queries on '
                             'terminal, one at a time, terms delimited by '
                             'spaces).')
    parser.add_argument('-b', '--batch', action='store_true',
                        help='Perform batch of queries (read queries from '
                             'standard input, one per line, terms delimited '
                             'by spaces).')
    parser.add_argument('--with-scores', action='store_true',
                        help='Print score next to each hit (separated by a '
                             'tab).')
    parser.add_argument('--about', action='store_true',
                        help='Print output of searchService.about()')
    parser.add_argument('--alive', action='store_true',
                        help='Print output of searchService.alive()')
    parser.add_argument('--capabilities', action='store_true',
                        help='Print output of searchService.getCapabilities()')
    parser.add_argument('--corpora', action='store_true',
                        help='Print output of searchService.getCorpora()')
    parser.add_argument('-l', '--loglevel', '--log-level',
                        help='Logging verbosity level threshold (to stderr)',
                        default='info')
    parser.add_argument('terms', metavar='term', nargs='*', type=unicode_arg,
                        help='Single query to perform (mutually exclusive with -b -and -i)')
    concrete.version.add_argparse_argument(parser)
    args = parser.parse_args()

    logging.basicConfig(format='%(asctime)-15s %(levelname)s: %(message)s',
                        level=args.loglevel.upper())

    with SearchClientWrapper(args.host, args.port) as search_client:

        if args.about:
            print("SearchService.about() returned %s" % search_client.about())
        if args.alive:
            print("SearchService.alive() returned %s" % search_client.alive())
        if args.capabilities:
            print("SearchService.getCapabilities() returned %s" % search_client.getCapabilities())
        if args.corpora:
            print("SearchService.getCorpora() returned %s" % search_client.getCorpora())

        if args.interactive:
            if args.batch:
                raise Exception(
                    'batch mode (-b) cannot be specified for '
                    'interactive client (-i)')
            if args.terms:
                raise Exception(
                    'query terms cannot be specified on command line for '
                    'interactive client (-i)')
            logging.info('starting interactive search client...')
            logging.info('note: query terms are delimited by tabs')
            logging.info('enter a blank query to exit')
            line = raw_input('> ')
            while line:
                terms = unicode_arg(line).split('\t')
                for (comm_id, score) in execute_search_query(search_client, terms, args.k):
                    if args.with_scores:
                        print('{}	{}'.format(comm_id, score))
                    else:
                        print(comm_id)
                line = raw_input('> ')
        elif args.batch:
            logging.info('starting batch non-interactive search client...')
            if args.terms:
                raise Exception(
                    'query terms cannot be specified on command line for '
                    'batch client (-b)')
            logging.info('note: query terms are delimited by tabs')
            for line in sys.stdin:
                terms = line.rstrip('\r\n').split('\t')
                print('\t'.join(
                    (
                        '{} {}'.format(comm_id, score)
                        if args.with_scores else comm_id
                    )
                    for (comm_id, score)
                    in execute_search_query(search_client, terms, args.k)
                ))
        else:
            logging.info('starting single-query non-interactive search client...')
            if args.terms:
                for (comm_id, score) in execute_search_query(search_client, args.terms, args.k):
                    if args.with_scores:
                        print('{}	{}'.format(comm_id, score))
                    else:
                        print(comm_id)
            else:
                logging.warning('No search terms specified')


if __name__ == '__main__':
    main()
